"""
Main CLI module for pykickstart.
"""

import os
import sys
import logging
import subprocess
from pathlib import Path
from typing import Optional

import click
import questionary
from rich.console import Console
from rich.logging import RichHandler

from .venv_utils import create_venv, activate_venv
from .reqs_handler import handle_requirements
from .shell_utils import detect_shell
from .project_utils import setup_project_structure

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
    handlers=[
        RichHandler(rich_tracebacks=True),
        logging.FileHandler(".pykickstart.log")
    ]
)

logger = logging.getLogger("pykickstart")
console = Console()

@click.group()
@click.version_option()
def cli():
    """Pykickstart - Python project environment setup automation tool."""
    pass

@cli.command()
@click.option("--venv-name", default=".venv", help="Name of the virtual environment directory")
@click.option("--python", default=None, help="Python interpreter to use")
@click.option("--no-git", is_flag=True, help="Skip Git initialization")
def init(venv_name: str, python: Optional[str], no_git: bool):
    """Initialize a new Python project environment."""
    try:
        # Detect shell
        shell = detect_shell()
        console.print(f"📝 Detected shell: {shell}", style="bold green")

        # Create virtual environment
        venv_path = create_venv(venv_name, python)
        console.print(f"✨ Created virtual environment at {venv_path}", style="bold green")

        # Activate virtual environment
        activate_venv(venv_path, shell)
        console.print("🔌 Activated virtual environment", style="bold green")

        # Handle requirements
        handle_requirements()

        # Setup project structure
        if questionary.confirm("Would you like to set up a basic project structure?").ask():
            setup_project_structure()

        # Initialize Git repository
        if not no_git and questionary.confirm("Would you like to initialize a Git repository?").ask():
            subprocess.run(["git", "init"], check=True)
            console.print("📦 Initialized Git repository", style="bold green")

        console.print("\n✅ Ready to code!", style="bold green")

    except Exception as e:
        logger.error(f"An error occurred: {str(e)}")
        sys.exit(1)

def main():
    """Entry point for the CLI."""
    cli() 