"""
Project structure and utility functions.
"""

import os
from pathlib import Path

def setup_project_structure() -> None:
    """
    Create a basic project structure with common directories and files.
    """
    # Create directories
    directories = [
        "src",
        "tests",
        "docs",
        ".github/workflows"
    ]
    
    for directory in directories:
        Path(directory).mkdir(parents=True, exist_ok=True)

    # Create .gitignore
    gitignore_content = """# Python
__pycache__/
*.py[cod]
*$py.class
*.so
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg

# Virtual Environment
.env
.venv
env/
venv/
ENV/

# IDE
.idea/
.vscode/
*.swp
*.swo

# Testing
.coverage
htmlcov/
.pytest_cache/

# Logs
*.log
"""
    with open(".gitignore", "w", encoding="utf-8") as f:
        f.write(gitignore_content)

    # Create basic README.md if it doesn't exist
    if not os.path.exists("README.md"):
        readme_content = """# Project Name

Brief description of your project.

## Installation

```bash
pip install -r requirements.txt
```

## Usage

Describe how to use your project.

## Development

1. Clone the repository
2. Install dependencies: `pip install -r requirements.txt`
3. Run tests: `pytest`

## License

MIT License
"""
        with open("README.md", "w", encoding="utf-8") as f:
            f.write(readme_content)

    # Create basic test file
    test_content = """import pytest

def test_example():
    assert True
"""
    with open("tests/test_example.py", "w", encoding="utf-8") as f:
        f.write(test_content)

    # Create GitHub Actions workflow for CI
    workflow_content = """name: CI

on:
  push:
    branches: [ main ]
  pull_request:
    branches: [ main ]

jobs:
  test:
    runs-on: ubuntu-latest
    strategy:
      matrix:
        python-version: [3.8, 3.9, '3.10']

    steps:
    - uses: actions/checkout@v2
    - name: Set up Python ${{ matrix.python-version }}
      uses: actions/setup-python@v2
      with:
        python-version: ${{ matrix.python-version }}
    - name: Install dependencies
      run: |
        python -m pip install --upgrade pip
        pip install -r requirements.txt
    - name: Run tests
      run: |
        pytest
"""
    workflow_path = Path(".github/workflows/ci.yml")
    workflow_path.parent.mkdir(parents=True, exist_ok=True)
    with open(workflow_path, "w", encoding="utf-8") as f:
        f.write(workflow_content) 