"""
Requirements management utilities.
"""

import os
import subprocess
from pathlib import Path
import google.generativeai as genai
from rich.console import Console

console = Console()

def handle_requirements() -> None:
    """
    Handle project requirements:
    1. Check for existing requirements.txt
    2. If not found, scan Python files and create requirements.txt
    3. Install dependencies
    """
    if os.path.exists("requirements.txt"):
        console.print("📦 Found requirements.txt, installing dependencies...", style="bold blue")
        subprocess.run([sys.executable, "-m", "pip", "install", "-r", "requirements.txt"], check=True)
        return

    console.print("🔍 No requirements.txt found, scanning project files...", style="bold blue")
    
    try:
        # Try using pipreqs first
        subprocess.run(["pipreqs", ".", "--force"], check=True)
        console.print("✨ Generated requirements.txt using pipreqs", style="bold green")
    except (subprocess.SubprocessError, FileNotFoundError):
        console.print("⚠️ pipreqs failed, falling back to AI-based analysis...", style="bold yellow")
        generate_requirements_with_ai()

    # Install the generated requirements
    if os.path.exists("requirements.txt"):
        subprocess.run([sys.executable, "-m", "pip", "install", "-r", "requirements.txt"], check=True)
        console.print("✅ Dependencies installed successfully", style="bold green")

def generate_requirements_with_ai() -> None:
    """
    Use Google's Gemini AI to analyze Python files and generate requirements.txt.
    """
    # Initialize Gemini
    genai.configure(api_key=os.getenv("GOOGLE_API_KEY"))
    model = genai.GenerativeModel('gemini-pro')

    # Collect all Python files
    python_files = list(Path(".").rglob("*.py"))
    if not python_files:
        console.print("⚠️ No Python files found in the project", style="bold yellow")
        return

    # Read and analyze Python files
    imports = set()
    for file in python_files:
        with open(file, "r", encoding="utf-8") as f:
            content = f.read()
            # Use Gemini to analyze imports
            response = model.generate_content(
                f"Analyze this Python code and list all external package imports (excluding standard library):\n\n{content}"
            )
            if response.text:
                imports.update(response.text.strip().split("\n"))

    # Write requirements.txt
    with open("requirements.txt", "w", encoding="utf-8") as f:
        for imp in sorted(imports):
            f.write(f"{imp}\n")

    console.print("✨ Generated requirements.txt using AI analysis", style="bold green")

def suggest_dev_tools() -> None:
    """
    Suggest and optionally install common development tools.
    """
    dev_tools = {
        "black": "Code formatter",
        "flake8": "Linter",
        "pytest": "Testing framework",
        "mypy": "Static type checker",
        "isort": "Import sorter"
    }

    choices = questionary.checkbox(
        "Would you like to install any of these development tools?",
        choices=[f"{tool} - {desc}" for tool, desc in dev_tools.items()]
    ).ask()

    if choices:
        tools = [choice.split(" - ")[0] for choice in choices]
        subprocess.run([sys.executable, "-m", "pip", "install"] + tools, check=True)
        console.print("✅ Development tools installed successfully", style="bold green") 