"""
Virtual environment management utilities.
"""

import os
import sys
import subprocess
from pathlib import Path
from typing import Optional

def create_venv(venv_name: str, python: Optional[str] = None) -> Path:
    """
    Create a new virtual environment.
    
    Args:
        venv_name: Name of the virtual environment directory
        python: Python interpreter to use (optional)
    
    Returns:
        Path to the created virtual environment
    """
    venv_path = Path(venv_name)
    
    if venv_path.exists():
        raise FileExistsError(f"Virtual environment directory {venv_name} already exists")
    
    cmd = [sys.executable, "-m", "venv", str(venv_path)]
    if python:
        cmd[0] = python
    
    subprocess.run(cmd, check=True)
    return venv_path

def activate_venv(venv_path: Path, shell: str) -> None:
    """
    Activate the virtual environment in the current shell.
    
    Args:
        venv_path: Path to the virtual environment
        shell: Shell type (bash, zsh, etc.)
    """
    if os.name == "nt":  # Windows
        activate_script = venv_path / "Scripts" / "activate.bat"
        if not activate_script.exists():
            raise FileNotFoundError(f"Activation script not found at {activate_script}")
        
        # On Windows, we need to modify the current process environment
        os.environ["VIRTUAL_ENV"] = str(venv_path)
        os.environ["PATH"] = f"{venv_path}\\Scripts;{os.environ['PATH']}"
        
    else:  # Unix-like
        activate_script = venv_path / "bin" / "activate"
        if not activate_script.exists():
            raise FileNotFoundError(f"Activation script not found at {activate_script}")
        
        # Source the activation script
        subprocess.run(f"source {activate_script}", shell=True, check=True) 